import os

from flask import Flask, g
# from flask.ext.bootstrap import Bootstrap
from flask_bootstrap import Bootstrap
import logging
import numpy as np
# @app.route('/', methods=('GET', 'POST'))
# def load_logged_in_user():
#     user_id = session.get('user_id')

#     if user_id is None:
#         g.user = None
#     else:
#         g.user = user_id
from flask_wtf import FlaskForm
from wtforms import SubmitField
class submit_form(FlaskForm):
    submit_button = SubmitField('Next')

bootstrap = Bootstrap()
from flaskr.util import task_ID_list, load_loan_data
loan_task_list = load_loan_data()

from flaskr.db import get_db, get_engine
def save_user_info(user_id, user_task_order, reverse_flag_list, user_group, analogy_type):
    from flaskr.db import UserInfo
    db = get_db()
    q = db.query(UserInfo.user_id).filter(UserInfo.user_id==user_id)
    exist_flag = db.query(q.exists()).scalar()
    if exist_flag:
        return True
    task_order_str = "|".join(user_task_order)
    reverse_flag_str = "|".join(reverse_flag_list)

    ui_obj = UserInfo(user_id=user_id, task_order_str=task_order_str, reverse_flag_str=reverse_flag_str, user_group=user_group, analogy_type=analogy_type)

    db.add(ui_obj)
    db.commit()
    return False

def get_loan_data(task_id = 'LP001002'):
    # engine = get_engine()
    # cmd = 'SELECT * FROM loan WHERE Loan_ID = :task_id'
    return loan_task_list[task_id].serize()

def create_app(test_config=None):
    # create and configure the app
    app = Flask(__name__, instance_relative_config=True)
    app.config.from_mapping(
        SECRET_KEY='', # setting this attribute, but don't share with others
        DATABASE=os.path.join(app.instance_path, 'flaskr.sqlite'),
        BOOTSTRAP_BOOTSWATCH_THEME='cerulean'
    )
    handler = logging.FileHandler('flask.log', encoding='UTF-8')
    handler.setLevel(logging.INFO)
    logging_format = logging.Formatter('%(asctime)s - %(levelname)s - %(filename)s - %(funcName)s - %(lineno)s - %(message)s')
    handler.setFormatter(logging_format)
    app.logger.addHandler(handler)
    # cerulean, materia
    # app.config['BOOTSTRAP_BOOTSWATCH_THEME'] = 'lumen'
    # Bootstrap(app)
    # for Flask Bootstrap

    bootstrap.init_app(app)
    # for Bootstrap-Flask

    if test_config is None:
        # load the instance config, if it exists, when not testing
        app.config.from_pyfile('config.py', silent=True)
    else:
        # load the test config if passed in
        app.config.from_mapping(test_config)

    # ensure the instance folder exists
    try:
        os.makedirs(app.instance_path)
    except OSError:
        pass

    from . import db
    db.init_app(app)

    from . import questionnaire
    app.register_blueprint(questionnaire.bp)

    from . import loan_task
    app.register_blueprint(loan_task.bp)

    from flask import redirect, url_for, session, render_template, request

    # a simple page that says hello
    @app.route('/hello')
    def hello():
        return 'Hello, World!'

    # @app.before_request
    # def load_logged_in_user():
    #     user_id = session.get('user_id')

    #     if user_id is None:
    #         g.user = None
    #     else:
    #         g.user = user_id
    # g.user_number = 0

    @app.route('/instruction', methods=('GET', 'POST'))
    def instruction():
        # show the user instruction for whole task
        form = submit_form()
        if request.method == 'POST':
            # we may record the time user finish reading instructions
            return redirect(url_for("questionnaire.preq"))
        return render_template('instruction.html', form=form)

    @app.route('/')
    def show_user_profile():
        user_id = request.args.get('PROLIFIC_PID')
        # To-do: recognize participated users, and ask them to leave
        # we may record the time user enter our web app
        # show the user profile for that user
        session['user_id'] = user_id
        session['task_id'] = 0
        session['attention_chck_right'] = 0
        # groups = ["analogy"]
        # user_group = np.random.choice(groups, 1)[0]
        session['user_group'] = "analogy"
        user_group = 'analogy'
        # session['user_group'] = "base"
        # analogy_types = ["vaccine", "weather", "train"]
        # analogy_type = np.random.choice(analogy_types, 1)[0]
        session['analogy_type'] = "train"
        analogy_type = 'train'
        # session['analogy_type'] = "weather"
        
        tp_task_ID_list = np.random.permutation(task_ID_list).tolist()
        session['task_ID_list'] = tp_task_ID_list
        reverse_flag_list = np.random.permutation(['True'] * 3 + ['False'] * 7).tolist()
        # True means reverse the advice from correct.
        session['reverse_flag_list'] = reverse_flag_list
        system_advice_list = []
        for i, task_id in enumerate(tp_task_ID_list):
            loan_data = get_loan_data(task_id)
            reverse_flag = eval(reverse_flag_list[i])
            if reverse_flag:
                # reverse the answer to show
                system_advice = "reject" if loan_data['Loan_Status'] == "Y" else "accept"
            else:
                system_advice = "accept" if loan_data['Loan_Status'] == "Y" else "reject"
            system_advice_list.append(system_advice)
        session['system_advice_list'] = system_advice_list
        exist_flag = save_user_info(user_id=user_id, user_task_order=tp_task_ID_list, reverse_flag_list=reverse_flag_list, user_group=user_group, analogy_type=analogy_type)
        if exist_flag:
            return render_template('thanks.html', reason_str = "You have participated this task once. We don't permit one person for multiple participation")
            app.logger.info("User {} come in for more than one time, show him the thanks page".format(user_id))
        # groups = ["base", "system", "accuracy", "analogy"]
        session['check_pass'] = False
        app.logger.info("User {} log in, with {} type of analogy, {} user group".format(user_id, session['analogy_type'], session['user_group']))
        # print("userid recorded as:", session['user_id'])
        # form = submit_form()
        # if request.method == 'POST' and form.validate_on_submit():
        # response = redirect(url_for("questionnaire.preq"))
        # return response
        # return render_template('instruction.html', form=form)
        return redirect(url_for("instruction"))
        # response = render_template('instruction.html')
        # response.set_cookie('user_id', user_id)
        # return response

    # @app.route('/user/<int:user_id>')
    # @app.route('/user/<username>')
    # @app.route('/prolifc_id/<user_id>')
    # def show_user_profile(user_id):
    #     # we may record the time user enter our web app
    #     # show the user profile for that user
    #     session['user_id'] = user_id
    #     return redirect(url_for("instruction"))

    return app
